﻿// *********************************************************************
// [DCOM Productions]
// [Copyright (C) DCOM Productions All rights reserved.]
// *********************************************************************

namespace BitFlex.EnterpriseServices.Input {
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms;

    /// <summary>
    /// Event Arguments for <see cref="HookService.MouseMove"/>
    /// </summary>
    public class MouseHookEventArgs : System.Windows.Forms.MouseEventArgs {
        /// <summary>
        /// Initializes a new instance of the BitFlex.EnterpriseServices.MouseHookEventArgs class
        /// </summary>
        /// <param name="button">One of the System.Windows.Forms.MouseButtons values indicating which mouse button was pressed</param>
        /// <param name="clicks">The number of times a mouse button was pressed</param>
        /// <param name="x">The x-coordinate of a mouse click, in pixels</param>
        /// <param name="y">The y-coordinate of a mouse click, in pixels</param>
        /// <param name="delta">A signed count of the number of detents the wheel has rotated</param>
        public MouseHookEventArgs(MouseButtons button, int clicks, int x, int y, int delta) 
            : base(button, clicks, x, y, delta) {
            // No additional constructor code required
        }

        /// <summary>
        /// Initializes a new instance of the BitFlex.EnterpriseServices.MouseHookEventArgs class
        /// </summary>
        /// <param name="e">A <see cref="System.Windows.Forms.MouseEventArgs"/> object that defines the mouse properties</param>
        public MouseHookEventArgs(MouseEventArgs e) 
            : this(e.Button, e.Clicks, e.X, e.Y, e.Delta) {
            // No additional constructor code required
        }

        private bool m_Handled = false;
        /// <summary>
        /// Gets or sets a System.Boolean value indicating whether the mouse events will be processed for other applications (globally)
        /// </summary>
        public bool Handled {
            get {
                return m_Handled;
            }
            set {
                m_Handled = value;
            }
        }
    }
}