﻿// *********************************************************************
// [DCOM Productions]
// [Copyright (C) DCOM Productions All rights reserved.]
// *********************************************************************

namespace BitFlex.Win32.Constants {
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Defines the constant values that represent various Application Desktop Toolbar messages sent and received by the Windows Shell
    /// </summary>
    public enum ABMessage {
        /// <summary>
        /// Registers a new appbar and specifies the message identifier that the system should use to send it notification messages. An appbar should send this message before sending any other appbar messages.
        /// </summary>
        ABM_NEW = 0x00000000,

        /// <summary>
        /// Unregisters an appbar by removing it from the system's internal list. The system no longer sends notification messages to the appbar or prevents other applications from using the screen area occupied by the appbar.
        /// </summary>
        ABM_REMOVE = 0x00000001,

        /// <summary>
        /// Requests a size and screen position for an appbar. When the request is made, the message proposes a screen edge and a bounding rectangle for the appbar. The system adjusts the bounding rectangle so that the appbar does not interfere with the Windows taskbar or any other appbars.
        /// </summary>
        ABM_QUERYPOS = 0x00000002,

        /// <summary>
        /// Sets the size and screen position of an appbar. The message specifies a screen edge and the bounding rectangle for the appbar. The system may adjust the bounding rectangle so that the appbar does not interfere with the Windows taskbar or any other appbars.
        /// </summary>
        ABM_SETPOS = 0x00000003,

        /// <summary>
        /// Retrieves the autohide and always-on-top states of the Windows taskbar.
        /// </summary>
        ABM_GETSTATE = 0x00000004,

        /// <summary>
        /// Retrieves the bounding rectangle of the Windows taskbar.
        /// </summary>
        ABM_GETTASKBARPOS = 0x00000005,

        /// <summary>
        /// Notifies the system that an appbar has been activated. An appbar should call this message in response to the WM_ACTIVATE message.
        /// </summary>
        /// <remarks>lParam, TRUE means activate, FALSE means deactivate</remarks>
        ABM_ACTIVATE = 6,

        /// <summary>
        /// Retrieves the handle to the autohide appbar associated with an edge of the screen.
        /// </summary>
        ABM_GETAUTOHIDEBAR = 0x00000007,

        /// <summary>
        /// Registers or unregisters an autohide appbar for an edge of the screen.
        /// </summary>
        /// <remarks>This can fail at any time. You MUST check the result. lParam, TRUE means set, false means unset. uEdge is which edge.</remarks>
        ABM_SETAUTOHIDEBAR = 0x00000008,

        /// <summary>
        /// Notifies the system when an appbar's position has changed. An appbar should call this message in response to the WM_WINDOWPOSCHANGED message.
        /// </summary>
        ABM_WINDOWPOSCHANGED = 0x00000009,

        /// <summary>
        /// Sets the autohide and always-on-top states of the Windows taskbar.
        /// </summary>
        ABM_SETSTATE = 0x0000000a
    }
}