﻿// *********************************************************************
// [DCOM Productions]
// [Copyright (C) DCOM Productions All rights reserved.]
// *********************************************************************

namespace BitFlex.Win32.Structures {
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Runtime.InteropServices;

    /// <summary>
    /// The RECT structure defines the coordinates of the upper-left and lower-right corners of a rectangle.
    /// </summary>
    /// <devdoc>
    ///     <para>
    ///         typedef struct _RECT {
    ///           LONG left;
    ///           LONG top;
    ///           LONG right;
    ///           LONG bottom;
    ///         } RECT, *PRECT;
    ///     </para>
    /// </devdoc>
    /// <remarks>
    /// By convention, the right and bottom edges of the rectangle are normally considered exclusive. In other words, the pixel 
    /// whose coordinates are ( right, bottom ) lies immediately outside of the rectangle. For example, when RECT is passed to 
    /// the FillRect function, the rectangle is filled up to, but not including, the right column and bottom row of pixels. 
    /// This structure is identical to the RECTL structure.
    /// </remarks>
    [StructLayout(LayoutKind.Sequential)]
    public struct RECT {
        public int left;
        public int top;
        public int right;
        public int bottom;
    }
}
