﻿// *********************************************************************
// [DCOM Productions]
// [Copyright (C) DCOM Productions All rights reserved.]
// *********************************************************************

namespace BitFlex.Windows.Forms.Design {
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;
    using System.Windows.Forms.Design;
    using System.Drawing.Drawing2D;
    using System.Drawing;
    using System.ComponentModel.Design;

    /// <summary>
    /// Windows Forms Designer for <see cref="BitFlex.Windows.Forms.WizardPage"/>
    /// </summary>
    public class WizardPageDesigner : ParentControlDesigner {

        private WizardPageActionList m_ActionList;
        /// <summary>
        /// Overrides base implementation of ActionLists
        /// </summary>
        public override System.ComponentModel.Design.DesignerActionListCollection ActionLists {
            get {
                if (m_ActionList == null) {
                    m_ActionList = new WizardPageActionList(this.Component);
                }
                return new DesignerActionListCollection(new DesignerActionList[] { m_ActionList });
            }
        }

        protected override void OnPaintAdornments(System.Windows.Forms.PaintEventArgs pe) {
            base.OnPaintAdornments(pe);
            using (Pen pen = new Pen(Brushes.LightGray, 1.0F) { DashStyle = DashStyle.Dash }) {
                Rectangle rect = Control.ClientRectangle;
                rect.Height--;
                rect.Width--;
                pe.Graphics.DrawRectangle(pen, rect);
            }
            string name = Control.Name;  
            Size size = pe.Graphics.MeasureString(name, Control.Font).ToSize();
            int x = Control.ClientRectangle.Width - size.Width - Control.Padding.Right;
            if ((Control.Parent as Wizard) != null && (Control.Parent as Wizard).Watermark != null) {
                x -= (Control.Parent as Wizard).Watermark.Width + 4;
            }
            int y = Control.Padding.Top;
            PointF point = new PointF(x, y);
            pe.Graphics.DrawString(Control.Name, Control.Font, SystemBrushes.GrayText, point);
        }
    }
}