﻿// *********************************************************************
// [DCOM Productions]
// [Copyright (C) DCOM Productions All rights reserved.]
// *********************************************************************

namespace BitFlex.Xdf.Collections {
    using System;
    using System.Collections.Generic;
    using System.Linq;
    using System.Text;

    /// <summary>
    /// Base collection of <see cref="XdfKey"/> elements
    /// </summary>
    public class XdfKeyList : List<XdfKey> {
        /// <summary>
        /// Initializes a new instance of the BitFlex.Xdf.Collections.XdfKeyList class
        /// </summary>
        public XdfKeyList()
            : base() {
        }

        /// <summary>
        /// Initializes a new instance of the BitFlex.Xdf.Collections.XdfKeyList class
        /// </summary>
        public XdfKeyList(int capacity)
            : base(capacity) {
        }

        /// <summary>
        /// Initializes a new instance of the BitFlex.Xdf.Collections.XdfKeyList class
        /// </summary>
        public XdfKeyList(IEnumerable<XdfKey> collection)
            : base(collection) {
        }

        /// <summary>
        /// Gets or sets an XdfKey in the collection by name
        /// </summary>
        /// <param name="name">The name of the key</param>
        /// <returns>If using the Get accessor, returns the XdfKey whose name matches the name parameter</returns>
        public XdfKey this[string name] {
            get {
                return Find(s => s.Name.ToLower() == name.ToLower());
            }
            set {
                this[this.IndexOf(Find(s => s.Name.ToLower() == name.ToLower()))] = value;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="XdfSection"/> that owns this collection of keys
        /// </summary>
        public XdfSection Owner {
            get;
            set;
        }

        /// <summary>
        /// Adds the specified item to the collection
        /// </summary>
        /// <param name="item">The item to add</param>
        new public void Add(XdfKey item) {
            item.Parent = Owner;
            base.Add(item);
        }

        /// <summary>
        /// Adds the specified collection of keys to the collection
        /// </summary>
        /// <param name="collection">The collection to add</param>
        new public void AddRange(IEnumerable<XdfKey> collection) {
            foreach (XdfKey item in collection) {
                item.Parent = Owner;
            }
            base.AddRange(collection);
        }
    }
}
